/////////////////////////////////////////////////////////////////////////////
// Name:       
// Purpose:  
// Author:      Cesar Mauri Loba (cesar at crea-si dot com)
// Modified by: 
// Created:     
// Copyright:   (C) 2010 Cesar Mauri Loba - CREA Software Systems
// 
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
/////////////////////////////////////////////////////////////////////////////

// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

////@begin includes
#include "about.h"
////@end includes
#include <wx/config.h>

#include "mainwin.h"
#include "exceptions.h"
#include "ballonconfiguration.h"
#include "playwithvoicedialog.h"
#include "virtualinstrumentconfig.h"
#include "puredatawrapper.h"
#include "puredatatestw.h"
#include "cameraconfiguration.h"
#include "camwindow2.h"
#include "midiconfiguration.h"
#include "cameradialog2.h"

////@begin XPM images
#include "bitmaps/sitplus_logo_32x32.xpm"
////@end XPM images


/*!
 * MainWin type definition
 */

IMPLEMENT_CLASS( MainWin, wxMDIParentFrame )


/*!
 * MainWin event table definition
 */

BEGIN_EVENT_TABLE( MainWin, wxMDIParentFrame )

////@begin MainWin event table entries
    EVT_CLOSE( MainWin::OnCloseWindow )
    EVT_WINDOW_DESTROY( MainWin::OnDestroy )

    EVT_MENU( ID_MENU_EXIT, MainWin::OnMenuExitClick )

    EVT_MENU( ID_MENU_START_BALLON, MainWin::OnMenuStartBallonClick )

    EVT_MENU( ID_MENU_START_PVOICE, MainWin::OnMenuStartPvoiceClick )

    EVT_MENU( ID_MENUITEM_VIRTUAL_INSTRUMENT, MainWin::OnMenuitemVirtualInstrumentClick )

    EVT_MENU( ID_MENUITEM_CONFIG_AUDIO, MainWin::OnMenuitemConfigAudioClick )

    EVT_MENU( ID_MENUITEM_CAMERA, MainWin::OnMenuitemCameraClick )

    EVT_MENU( ID_MENUITEM_MIDI, MainWin::OnMenuitemMidiClick )

    EVT_MENU( ID_MENUITEM_ABOUT, MainWin::OnMenuitemAboutClick )

////@end MainWin event table entries

	EVT_COMMAND (wxID_ANY, wxEVT_CAMCTRL_FIRSTPROCESSOR, MainWin::OnShouldOpenCam )

	EVT_COMMAND (wxID_ANY, wxEVT_CAMCTRL_LASTPROCESSOR, MainWin::OnShouldCloseCam )
	
	EVT_COMMAND (wxID_ANY, wxEVT_CAMCTRL_ERROR, MainWin::OnCameraError )

	EVT_COMMAND (wxID_ANY, wxEVT_CAMCTRL_FATAL_ERROR, MainWin::OnCameraError )
	
END_EVENT_TABLE()


/*!
 * MainWin constructors
 */

MainWin::MainWin()
{
    Init();
}

MainWin::MainWin( wxWindow* parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
{
    Init();
    Create( parent, id, caption, pos, size, style );
}


/*!
 * MainWin creator
 */

bool MainWin::Create( wxWindow* parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
{
////@begin MainWin creation
    wxMDIParentFrame::Create( parent, id, caption, pos, size, style );

    CreateControls();
    SetIcon(GetIconResource(wxT("bitmaps/sitplus_logo_32x32.xpm")));
    Centre();
////@end MainWin creation	

    return true;
}


/*!
 * MainWin destructor
 */

MainWin::~MainWin()
{
////@begin MainWin destruction
////@end MainWin destruction
}


/*!
 * Member initialisation
 */

void MainWin::Init()
{	
////@begin MainWin member initialisation
    m_menuActivity = NULL;
    m_menuConfiguration = NULL;
////@end MainWin member initialisation
	m_camDialog= NULL;
}


/*!
 * Control creation for MainWin
 */

void MainWin::CreateControls()
{    
////@begin MainWin content construction
    MainWin* itemMDIParentFrame1 = this;

    wxMenuBar* menuBar = new wxMenuBar;
    wxMenu* itemMenu3 = new wxMenu;
    itemMenu3->Append(ID_MENU_EXIT, _("&Exit"), wxEmptyString, wxITEM_NORMAL);
    menuBar->Append(itemMenu3, _("&File"));
    m_menuActivity = new wxMenu;
    m_menuActivity->Append(ID_MENU_START_BALLON, _("Blow Up Ballon"), wxEmptyString, wxITEM_NORMAL);
    m_menuActivity->Append(ID_MENU_START_PVOICE, _("Playing with Voice"), wxEmptyString, wxITEM_NORMAL);
    m_menuActivity->Append(ID_MENUITEM_VIRTUAL_INSTRUMENT, _("Virtual Instrument"), wxEmptyString, wxITEM_NORMAL);
    menuBar->Append(m_menuActivity, _("Activity"));
    m_menuConfiguration = new wxMenu;
    m_menuConfiguration->Append(ID_MENUITEM_CONFIG_AUDIO, _("Audio"), wxEmptyString, wxITEM_NORMAL);
    m_menuConfiguration->Append(ID_MENUITEM_CAMERA, _("Camera"), wxEmptyString, wxITEM_NORMAL);
    m_menuConfiguration->Append(ID_MENUITEM_MIDI, _("MIDI"), wxEmptyString, wxITEM_NORMAL);
    menuBar->Append(m_menuConfiguration, _("Configuration"));
    wxMenu* itemMenu13 = new wxMenu;
    itemMenu13->Append(ID_MENUITEM_ABOUT, _("About..."), wxEmptyString, wxITEM_NORMAL);
    menuBar->Append(itemMenu13, _("Help"));
    itemMDIParentFrame1->SetMenuBar(menuBar);

    // Connect events and objects
    itemMDIParentFrame1->Connect(ID_MAINWIN, wxEVT_DESTROY, wxWindowDestroyEventHandler(MainWin::OnDestroy), NULL, this);
////@end MainWin content construction
}


/*!
 * Should we show tooltips?
 */

bool MainWin::ShowToolTips()
{
    return true;
}

/*!
 * Get bitmap resources
 */

wxBitmap MainWin::GetBitmapResource( const wxString& name )
{
    // Bitmap retrieval
////@begin MainWin bitmap retrieval
    wxUnusedVar(name);
    return wxNullBitmap;
////@end MainWin bitmap retrieval
}

/*!
 * Get icon resources
 */

wxIcon MainWin::GetIconResource( const wxString& name )
{
    // Icon retrieval
////@begin MainWin icon retrieval
    wxUnusedVar(name);
    if (name == _T("bitmaps/sitplus_logo_32x32.xpm"))
    {
        wxIcon icon(sitplus_logo_32x32);
        return icon;
    }
    return wxNullIcon;
////@end MainWin icon retrieval
}


bool MainWin::ProcessEvent(wxEvent& event)
{
	try {
		return wxMDIParentFrame::ProcessEvent (event);
	}
	catch (BaseException& e)
	{
		wxMessageDialog dlg(NULL, e.getMessage(), _("SITPLUS error"));
		dlg.ShowModal();
	}

	return true;
}


/*!
 * wxEVT_COMMAND_MENU_SELECTED event handler for ID_MENU_EXIT
 */

void MainWin::OnMenuExitClick( wxCommandEvent& event )
{
	Close();

	event.Skip (false);
}

/*!
 * wxEVT_CLOSE_WINDOW event handler for ID_MAINWIN
 */

void MainWin::OnCloseWindow( wxCloseEvent& event )
{
	if (event.CanVeto())
	{
		wxMessageDialog dlg(NULL, _("Are you sure you want to close the application?"), _("SITPLUS"), wxYES_NO | wxNO_DEFAULT | wxICON_EXCLAMATION);
		if (dlg.ShowModal()== wxID_YES) Destroy();
	}

    event.Skip(false);
}

/*!
 * wxEVT_DESTROY event handler for ID_MAINWIN
 */

void MainWin::OnDestroy( wxWindowDestroyEvent& event )
{
	// Note that Destroy events might come from other windows (e.g. MDI childs)
	// and so cleanup operations cannot be done here unless we check if the event
	// comes from this window which is unlikely. See wxWindowDestroyEvent help
	// for further details.

	event.Skip();
}

/*

void MainWin::StartActivity(const int number)
{
//	if (m_act[number]!= NULL) return;

	wxMDIChildFrame* win;
	try
	{
		switch(number)
		{
		case MainWin::BALLON_ACTIVITY:
			//m_act[number]= 
			win= CBallonConfiguration::getInstance(this);			
			break;
		case MainWin::PVOICE_ACTIVITY:
			win= PlayWithVoiceDialog::getInstance(this);
			//MDIChildContainer* mdiChild= new MDIChildContainer (this);
			//mdiChild->AddPanel (new PlayWithVoiceDialog (mdiChild));			
			break;
			
		default:
			throw InitializationException(_("Internal error: no such activity"));
		}

		//m_act[number]->Show();
		win->Show();
		
//		m_menuActivity->Enable (ID_MENU_START_BALLON, false);
//		m_menuActivity->Enable (ID_MENU_START_PVOICE, false);
//		m_menuConfiguration->Enable (ID_MENUITEM_TEST_CONFIG_AUDIO, false);
//		m_menuConfiguration->Enable (ID_MENU_SELECTCAMERA, false);
		
	}
	catch (InitializationException& e)
	{
		//StopActivities();
		wxMessageDialog dlg(NULL, e.getMessage(), _("Initialization exception"));
		dlg.ShowModal();
	}
}
*/

/*!
 * wxEVT_COMMAND_MENU_SELECTED event handler for ID_MENUITEM
 */

void MainWin::OnMenuitemConfigAudioClick( wxCommandEvent& event )
{
	CPureDataTestW dlg(this);
	dlg.ShowModal ();
    event.Skip(false);
}

/*!
 * wxEVT_COMMAND_MENU_SELECTED event handler for ID_MENUITEM_CAMERA
 */

void MainWin::OnMenuitemCameraClick( wxCommandEvent& event )
{
	CCameraConfiguration* window = new CCameraConfiguration(this);
//	CCameraConfiguration window(this);// = new CCameraConfiguration(this);
//	window->Create2 (m_cameraController);
    window->ShowModal();
	window->Destroy();
 //   window->Destroy();

	event.Skip(false);
}


/*!
 * wxEVT_COMMAND_MENU_SELECTED event handler for ID_MENUITEM_MIDI
 */

void MainWin::OnMenuitemMidiClick( wxCommandEvent& event )
{
	CMIDIConfiguration cfgDialog (this);
	cfgDialog.ShowModal();

    event.Skip(false);
}


void MainWin::OnShouldOpenCam ( wxCommandEvent& event )
{
	assert (m_camDialog== NULL);

	m_menuConfiguration->Enable (ID_MENUITEM_CAMERA, false);
	//m_camDialog= new CameraDialog2(this);
	m_camDialog= new CCamDialog(this);
	m_camDialog->Show();

	event.Skip (false);
}

void MainWin::OnShouldCloseCam ( wxCommandEvent& event )
{
	assert (m_camDialog);
	m_camDialog->Close();
	m_camDialog= NULL;
	m_menuConfiguration->Enable (ID_MENUITEM_CAMERA, true);

	event.Skip (false);
}

void MainWin::OnCameraError ( wxCommandEvent& event )
{
	event.Skip (false);
}


/*!
 * wxEVT_COMMAND_MENU_SELECTED event handler for ID_MENU_START_BALLON
 */

void MainWin::OnMenuStartBallonClick( wxCommandEvent& event )
{
	CBallonConfiguration* win= CBallonConfiguration::getInstance(this);
	win->Show();
	
    event.Skip(false);	
}


/*!
 * wxEVT_COMMAND_MENU_SELECTED event handler for ID_MENU_START_PVOICE
 */

void MainWin::OnMenuStartPvoiceClick( wxCommandEvent& event )
{
	PlayWithVoiceDialog* win= PlayWithVoiceDialog::getInstance(this);
	win->Show();

    event.Skip(false);
}

/*!
 * wxEVT_COMMAND_MENU_SELECTED event handler for ID_MENUITEM_VIRTUAL_INSTRUMENT
 */

void MainWin::OnMenuitemVirtualInstrumentClick( wxCommandEvent& event )
{
	CVirtualInstrumentConfig* pvi= CVirtualInstrumentConfig::getInstance(this);
	pvi->Show();
	pvi->Move(10,10);

    event.Skip(false); 
}


/*!
 * wxEVT_COMMAND_MENU_SELECTED event handler for ID_MENUITEM_ABOUT
 */

void MainWin::OnMenuitemAboutClick( wxCommandEvent& event )
{
////@begin wxEVT_COMMAND_MENU_SELECTED event handler for ID_MENUITEM_ABOUT in MainWin.
    // Before editing this code, remove the block markers.
    About* window = new About(this);
    int returnValue = window->ShowModal();
    window->Destroy();
////@end wxEVT_COMMAND_MENU_SELECTED event handler for ID_MENUITEM_ABOUT in MainWin. 
}

