/////////////////////////////////////////////////////////////////////////////
// Name:       
// Purpose:  
// Author:      Cesar Mauri Loba (cesar at crea-si dot com)
// Modified by: 
// Created:     
// Copyright:   (C) 2010 Cesar Mauri Loba - CREA Software Systems
// 
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
/////////////////////////////////////////////////////////////////////////////
#include "midicontroller.h"

#define MAX_MIDI_OUT_DEVICES 20

#define ID_NO_DEVICE -1

CMIDIController* CMIDIController::g_instance= NULL;

CMIDIController* CMIDIController::getInstance(wxConfigBase* configBase)
{
	if (g_instance== NULL)
	{
		g_instance= new CMIDIController (configBase);
	}
	return g_instance;
}

void CMIDIController::destroyInstance()
{
	if (g_instance)
	{
		delete g_instance;
		g_instance= NULL;
	}

	delete wxMidiSystem::GetInstance();
}

CMIDIController::CMIDIController(wxConfigBase* configBase)
: m_configBase(configBase)
, m_outDevNumber(ID_NO_DEVICE)
{
	assert (m_configBase);

	m_configPath= m_configBase->GetPath();
}

CMIDIController::~CMIDIController(void)
{
}

#define MIDI_CONFIG_KEY _T("defaultMIDIOut")

void CMIDIController::LetChooseOutDevice()
{	
	wxString prevPath (m_configBase->GetPath());
	m_configBase->SetPath (m_configPath);
	m_configBase->DeleteEntry (MIDI_CONFIG_KEY);	
	m_configBase->SetPath (prevPath);
	m_configBase->Flush();
	m_outDevNumber= ID_NO_DEVICE;
	m_outDevName.Clear();
}

// -1 if not found
int CMIDIController::SearchOutDevice (const wxString& name)
{
	int retval= -1;
	int nNumDevices = wxMidiSystem::GetInstance()->CountDevices();	

	for (int i = 0; retval== -1 && i< nNumDevices; i++) {
      	wxMidiOutDevice* pMidiDev = new wxMidiOutDevice(i);
      	if (pMidiDev->IsOutputPort() && pMidiDev->DeviceName()== name) retval= i;
		delete pMidiDev;
   	}
	return retval;
}

void CMIDIController::SelectOutputDevice(wxString& name, int& number)
{
	wxArrayString devs;
	int numbers[MAX_MIDI_OUT_DEVICES], nOutput= 0;

	int nNumDevices = wxMidiSystem::GetInstance()->CountDevices();	
	for (int i = 0; i < nNumDevices && nOutput< MAX_MIDI_OUT_DEVICES; i++) {
      	wxMidiOutDevice* pMidiDev = new wxMidiOutDevice(i);
		if (pMidiDev->IsOutputPort()) {
			devs.Add (pMidiDev->DeviceName());      	
			numbers[nOutput++]= i;
		}
		delete pMidiDev;
   	}

	if (nOutput== 0) throw RunTimeException (_("No MIDI out devices detected"));

	wxSingleChoiceDialog choiceDlg(NULL, _("Choose the MIDI out device to use"), 
		_("MIDI subsystem"), devs, NULL, wxDEFAULT_DIALOG_STYLE | wxOK | wxCANCEL | wxCENTRE);

	if (choiceDlg.ShowModal ()!= wxID_OK) throw RunTimeException (_("No MIDI out device selected"));

	name= choiceDlg.GetStringSelection ();
	number= numbers[choiceDlg.GetSelection()];
}

wxMidiOutDevice* CMIDIController::GetOutDevice()
{
	if (m_outDevNumber< 0) {
		wxString prevPath (m_configBase->GetPath());
		try {			
			m_configBase->SetPath (m_configPath);
						
			// Try to read device ID from stored settings
			m_configBase->Read (MIDI_CONFIG_KEY, &m_outDevName);
			m_outDevNumber= SearchOutDevice (m_outDevName);
			
			if (m_outDevNumber== ID_NO_DEVICE) {
				// Not found. Remove name
				m_configBase->DeleteEntry (MIDI_CONFIG_KEY);	
				
				// Let user choose
				SelectOutputDevice(m_outDevName, m_outDevNumber);
				
				// Here we've got a device. Save
				m_configBase->Write (MIDI_CONFIG_KEY, m_outDevName);

				m_configBase->SetPath (prevPath);
			}
		}
		catch(...) {
			m_configBase->SetPath (prevPath);
			throw;
		}
	}

	return new wxMidiOutDevice (m_outDevNumber);
}