/////////////////////////////////////////////////////////////////////////////
// Name:       
// Purpose:  
// Author:      Cesar Mauri Loba (cesar at crea-si dot com)
// Modified by: 
// Created:     
// Copyright:   (C) 2010 Cesar Mauri Loba - CREA Software Systems
// 
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
/////////////////////////////////////////////////////////////////////////////

// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

////@begin includes
////@end includes

#include <wx/msgdlg.h>
#include <wx/timer.h>
#include "puredatatestw.h"

#include "testpd.h"
#include "vumeter.h"
#include "puredatawrapper.h"

////@begin XPM images
////@end XPM images

#define TIMER_ID 12345
/*!
 * CPureDataTestW type definition
 */

IMPLEMENT_DYNAMIC_CLASS( CPureDataTestW, wxDialog )


/*!
 * CPureDataTestW event table definition
 */

BEGIN_EVENT_TABLE( CPureDataTestW, wxDialog )

////@begin CPureDataTestW event table entries
    EVT_CLOSE( CPureDataTestW::OnCloseWindow )
    EVT_WINDOW_DESTROY( CPureDataTestW::OnDestroy )

    EVT_SPINCTRL( ID_SPINCTRL_DELAY, CPureDataTestW::OnSpinctrlDelayUpdated )

    EVT_RADIOBUTTON( ID_RADIOBUTTON_PLAYSOUND, CPureDataTestW::OnRadiobuttonPlaysoundSelected )

    EVT_RADIOBUTTON( ID_RADIOBUTTON_MIC, CPureDataTestW::OnRadiobuttonMicSelected )

    EVT_SLIDER( ID_SLIDER_MICVOL, CPureDataTestW::OnSliderMicvolUpdated )

    EVT_SLIDER( ID_SLIDER_OUTVOL, CPureDataTestW::OnSliderOutvolUpdated )

    EVT_CHECKBOX( ID_CHECKBOX_PD_GUI, CPureDataTestW::OnCheckboxPdGuiClick )

    EVT_CHECKBOX( ID_CHECKBOX_DEBUG_CONSOLE, CPureDataTestW::OnCheckboxDebugConsoleClick )

    EVT_BUTTON( ID_BUTTON_CLOSE, CPureDataTestW::OnButtonCloseClick )

////@end CPureDataTestW event table entries

	EVT_TIMER(TIMER_ID, CPureDataTestW::OnTimer)

END_EVENT_TABLE()


/*!
 * CPureDataTestW constructors
 */

CPureDataTestW::CPureDataTestW()
: m_timer(this, TIMER_ID)
{
    Init();
}

CPureDataTestW::CPureDataTestW( wxWindow* parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
: m_timer(this, TIMER_ID)
{
    Init();
    Create(parent, id, caption, pos, size, style);
}


/*!
 * CPureDataTestW creator
 */

bool CPureDataTestW::Create( wxWindow* parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
{
////@begin CPureDataTestW creation
    SetExtraStyle(wxWS_EX_BLOCK_EVENTS);
    wxDialog::Create( parent, id, caption, pos, size, style );

    CreateControls();
    if (GetSizer())
    {
        GetSizer()->SetSizeHints(this);
    }
    Centre();
////@end CPureDataTestW creation
    return true;
}


/*!
 * CPureDataTestW destructor
 */

CPureDataTestW::~CPureDataTestW()
{
////@begin CPureDataTestW destruction
////@end CPureDataTestW destruction
}


/*!
 * Member initialisation
 */

void CPureDataTestW::Init()
{
////@begin CPureDataTestW member initialisation
    m_spinDelay = NULL;
    m_txtMicVol = NULL;
    m_sldMicVol = NULL;
    m_gaugeMicInput = NULL;
    m_sldOutVol = NULL;
    m_gaugeOutput = NULL;
    m_chkGUIDebug = NULL;
    m_chkDebugConsole = NULL;
////@end CPureDataTestW member initialisation
//	m_activity= NULL;
	m_micTest= false;
}


/*!
 * Control creation for CPureDataTestW
 */

void CPureDataTestW::CreateControls()
{    
////@begin CPureDataTestW content construction
    CPureDataTestW* itemDialog1 = this;

    wxBoxSizer* itemBoxSizer2 = new wxBoxSizer(wxVERTICAL);
    itemDialog1->SetSizer(itemBoxSizer2);

    wxStaticText* itemStaticText3 = new wxStaticText;
    itemStaticText3->Create( itemDialog1, wxID_STATIC, _("Set \"Delay\" to the minimum\npossible value before you\nget sound defects."), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer2->Add(itemStaticText3, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);

    wxBoxSizer* itemBoxSizer4 = new wxBoxSizer(wxHORIZONTAL);
    itemBoxSizer2->Add(itemBoxSizer4, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);

    wxStaticText* itemStaticText5 = new wxStaticText;
    itemStaticText5->Create( itemDialog1, wxID_STATIC, _("Delay"), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer4->Add(itemStaticText5, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    m_spinDelay = new wxSpinCtrl;
    m_spinDelay->Create( itemDialog1, ID_SPINCTRL_DELAY, _T("1"), wxDefaultPosition, wxDefaultSize, wxSP_ARROW_KEYS, 1, 100, 1 );
    itemBoxSizer4->Add(m_spinDelay, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxStaticBox* itemStaticBoxSizer7Static = new wxStaticBox(itemDialog1, wxID_ANY, _("Test type"));
    wxStaticBoxSizer* itemStaticBoxSizer7 = new wxStaticBoxSizer(itemStaticBoxSizer7Static, wxVERTICAL);
    itemBoxSizer2->Add(itemStaticBoxSizer7, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);

    wxRadioButton* itemRadioButton8 = new wxRadioButton;
    itemRadioButton8->Create( itemDialog1, ID_RADIOBUTTON_PLAYSOUND, _("Play sound"), wxDefaultPosition, wxDefaultSize, 0 );
    itemRadioButton8->SetValue(true);
    itemStaticBoxSizer7->Add(itemRadioButton8, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);

    wxRadioButton* itemRadioButton9 = new wxRadioButton;
    itemRadioButton9->Create( itemDialog1, ID_RADIOBUTTON_MIC, _("Microphone"), wxDefaultPosition, wxDefaultSize, 0 );
    itemRadioButton9->SetValue(false);
    itemStaticBoxSizer7->Add(itemRadioButton9, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);

    wxBoxSizer* itemBoxSizer10 = new wxBoxSizer(wxHORIZONTAL);
    itemBoxSizer2->Add(itemBoxSizer10, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);

    m_txtMicVol = new wxStaticText;
    m_txtMicVol->Create( itemDialog1, wxID_STATIC, _("Mic. vol."), wxDefaultPosition, wxDefaultSize, 0 );
    m_txtMicVol->Enable(false);
    itemBoxSizer10->Add(m_txtMicVol, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    m_sldMicVol = new wxSlider;
    m_sldMicVol->Create( itemDialog1, ID_SLIDER_MICVOL, 0, 0, 100, wxDefaultPosition, wxDefaultSize, wxSL_HORIZONTAL );
    m_sldMicVol->Enable(false);
    itemBoxSizer10->Add(m_sldMicVol, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    m_gaugeMicInput = new wxGauge;
    m_gaugeMicInput->Create( itemDialog1, ID_GAUGE_MICINPUT, 100, wxDefaultPosition, wxSize(-1, 15), wxGA_HORIZONTAL );
    m_gaugeMicInput->SetValue(1);
    itemBoxSizer2->Add(m_gaugeMicInput, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);

    wxBoxSizer* itemBoxSizer14 = new wxBoxSizer(wxHORIZONTAL);
    itemBoxSizer2->Add(itemBoxSizer14, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);

    wxStaticText* itemStaticText15 = new wxStaticText;
    itemStaticText15->Create( itemDialog1, wxID_STATIC, _("Out. vol."), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer14->Add(itemStaticText15, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    m_sldOutVol = new wxSlider;
    m_sldOutVol->Create( itemDialog1, ID_SLIDER_OUTVOL, 0, 0, 100, wxDefaultPosition, wxDefaultSize, wxSL_HORIZONTAL );
    itemBoxSizer14->Add(m_sldOutVol, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    m_gaugeOutput = new wxGauge;
    m_gaugeOutput->Create( itemDialog1, ID_GAUGE_OUTVOL, 100, wxDefaultPosition, wxSize(-1, 15), wxGA_HORIZONTAL );
    m_gaugeOutput->SetValue(1);
    itemBoxSizer2->Add(m_gaugeOutput, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);

    wxCollapsiblePane* itemCollapsiblePane18 = new wxCollapsiblePane;
    itemCollapsiblePane18->Create( itemDialog1, ID_COLLAPSIBLEPANE, _("Advanced"), wxDefaultPosition, wxDefaultSize, wxCP_DEFAULT_STYLE );
    itemBoxSizer2->Add(itemCollapsiblePane18, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);

    wxBoxSizer* itemBoxSizer19 = new wxBoxSizer(wxVERTICAL);
    itemCollapsiblePane18->GetPane()->SetSizer(itemBoxSizer19);

    m_chkGUIDebug = new wxCheckBox;
    m_chkGUIDebug->Create( itemCollapsiblePane18->GetPane(), ID_CHECKBOX_PD_GUI, _("Open PD GUI (only for testing!)"), wxDefaultPosition, wxDefaultSize, 0 );
    m_chkGUIDebug->SetValue(false);
    itemBoxSizer19->Add(m_chkGUIDebug, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);

    m_chkDebugConsole = new wxCheckBox;
    m_chkDebugConsole->Create( itemCollapsiblePane18->GetPane(), ID_CHECKBOX_DEBUG_CONSOLE, _("Debug console"), wxDefaultPosition, wxDefaultSize, 0 );
    m_chkDebugConsole->SetValue(false);
    itemBoxSizer19->Add(m_chkDebugConsole, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);

    wxBoxSizer* itemBoxSizer22 = new wxBoxSizer(wxHORIZONTAL);
    itemBoxSizer2->Add(itemBoxSizer22, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);

    wxButton* itemButton23 = new wxButton;
    itemButton23->Create( itemDialog1, ID_BUTTON_CLOSE, _("Close"), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer22->Add(itemButton23, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    // Connect events and objects
    itemDialog1->Connect(ID_CPUREDATATESTW, wxEVT_DESTROY, wxWindowDestroyEventHandler(CPureDataTestW::OnDestroy), NULL, this);
////@end CPureDataTestW content construction

	CPureDataWrapper::getInstance()->StartPD();
	CAudioProperties* ap= CPureDataWrapper::getInstance()->getAudioProperties();
	if (ap) {
		CPureDataWrapper::getInstance()->SetIntelligentASIOConfig (ap->GetDelay());
		m_spinDelay->SetValue (ap->GetDelay());
		delete ap;
	}
	else {
		m_spinDelay->Disable();
		m_chkDebugConsole->Disable();
	}


	//m_activity= new CTestPD (); //wxTHREAD_JOINABLE);
	//m_activity.Create ();
	m_activity.Start();  //>Run();
	
	m_sldMicVol->SetRange (m_activity.getMicInputControl()->getMin(), m_activity.getMicInputControl()->getMax());
	m_sldMicVol->SetValue (m_activity.getMicInputControl()->getValue());
	wxCommandEvent evt (wxEVT_COMMAND_SLIDER_UPDATED, ID_SLIDER_MICVOL);
	GetEventHandler()->ProcessEvent( evt );
	m_gaugeMicInput->SetRange (100);

	m_sldOutVol->SetRange (m_activity.getOutputControl()->getMin(), m_activity.getOutputControl()->getMax());
	m_sldOutVol->SetValue (m_activity.getOutputControl()->getValue());
	wxCommandEvent evt2 (wxEVT_COMMAND_SLIDER_UPDATED, ID_SLIDER_OUTVOL);
	GetEventHandler()->ProcessEvent( evt );
	m_gaugeOutput->SetRange (100);

	m_chkDebugConsole->SetValue (CPureDataWrapper::getInstance()->IsLogEnabled ());
	//CPureDataWrapper::getInstance()->getDebugGUIMode();
	m_chkGUIDebug->SetValue (CPureDataWrapper::getInstance()->getDebugGUIMode());
	
	m_timer.Start (33);
}


/*!
 * Should we show tooltips?
 */

bool CPureDataTestW::ShowToolTips()
{
    return true;
}

/*!
 * Get bitmap resources
 */

wxBitmap CPureDataTestW::GetBitmapResource( const wxString& name )
{
    // Bitmap retrieval
////@begin CPureDataTestW bitmap retrieval
    wxUnusedVar(name);
    return wxNullBitmap;
////@end CPureDataTestW bitmap retrieval
}

/*!
 * Get icon resources
 */

wxIcon CPureDataTestW::GetIconResource( const wxString& name )
{
    // Icon retrieval
////@begin CPureDataTestW icon retrieval
    wxUnusedVar(name);
    return wxNullIcon;
////@end CPureDataTestW icon retrieval
}

void CPureDataTestW::OnTimer(wxTimerEvent& event)
{
	if (m_micTest)	{
		//m_gaugeMicInput->SetValue ((int) Envelope2VUMeter(m_activity.getMicInputEnvelope()));
		//m_gaugeOutput->SetValue ((int) Envelope2VUMeter(m_activity.getOutputEnvelope()));
		m_gaugeMicInput->SetValue ((int) Envelope2Meter(m_activity.getMicInputEnvelope()));		
	}
	else
		m_gaugeMicInput->SetValue (0);		
	m_gaugeOutput->SetValue ((int) Envelope2Meter(m_activity.getOutputEnvelope()));
}

/*!
 * wxEVT_COMMAND_SPINCTRL_UPDATED event handler for ID_SPINCTRL_DELAY
 */

void CPureDataTestW::OnSpinctrlDelayUpdated( wxSpinEvent& event )
{	
	CPureDataWrapper::getInstance()->SetIntelligentASIOConfig (event.GetPosition());
    event.Skip(false);
}


/*!
 * wxEVT_COMMAND_RADIOBUTTON_SELECTED event handler for ID_RADIOBUTTON_PLAYSOUND
 */

void CPureDataTestW::OnRadiobuttonPlaysoundSelected( wxCommandEvent& event )
{
	if (m_micTest) {
        m_sldMicVol->Enable(false);
        m_gaugeMicInput->Enable(false);
        m_txtMicVol->Enable(false);
		m_gaugeMicInput->SetValue (0);
		m_gaugeOutput->SetValue (0);
		m_micTest= false;
		m_activity.setTestType (CTestPD::TEST_PLAY_SOUND);
	}
    event.Skip(false);
}


/*!
 * wxEVT_COMMAND_RADIOBUTTON_SELECTED event handler for ID_RADIOBUTTON_MIC
 */

void CPureDataTestW::OnRadiobuttonMicSelected( wxCommandEvent& event )
{
	if (!m_micTest) {
        m_sldMicVol->Enable();
        m_gaugeMicInput->Enable();
        m_txtMicVol->Enable();
		m_micTest= true;
		m_activity.setTestType (CTestPD::TEST_MICROPHONE);
	}
    event.Skip(false);
}


/*!
 * wxEVT_COMMAND_CHECKBOX_CLICKED event handler for ID_CHECKBOX_DEBUG_CONSOLE
 */

void CPureDataTestW::OnCheckboxDebugConsoleClick( wxCommandEvent& event )
{
	if (event.IsChecked())
		CPureDataWrapper::getInstance()->EnableLog();
	else
		CPureDataWrapper::getInstance()->DisableLog();
}


/*!
 * wxEVT_COMMAND_CHECKBOX_CLICKED event handler for ID_CHECKBOX_PD_GUI
 */

void CPureDataTestW::OnCheckboxPdGuiClick( wxCommandEvent& event )
{
	if (event.IsChecked()) {
		wxMessageDialog msg(this, _("This mode is intended only for debug purposes.\nAre you sure you want to start GUI debug mode?\n"), _("Enable GUI Debug Mode"), wxYES_NO);
		if (msg.ShowModal()== wxID_YES)	{
			CPureDataWrapper::getInstance()->setDebugGUIMode(true);
//			m_activity.NotifyPathDied ();
			Destroy();
		}
		else
			m_chkGUIDebug->SetValue (false);
	}
	else {
		wxMessageDialog msg(this, _("Disable GUI Debug Mode?"), _("Disable GUI Debug Mode"), wxYES_NO);
		if (msg.ShowModal()== wxID_YES)	{
			CPureDataWrapper::getInstance()->setDebugGUIMode(false);
//			m_activity.NotifyPathDied ();
			//CPureDataWrapper::getInstance()->StartPD();
			Destroy();
		}
		else
			m_chkGUIDebug->SetValue (true);
	}
}


/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON_OK
 */

void CPureDataTestW::OnButtonCloseClick( wxCommandEvent& event )
{
	Close();
	
    event.Skip(false);
}


/*!
 * wxEVT_DESTROY event handler for ID_CPUREDATATESTW
 */

void CPureDataTestW::OnDestroy( wxWindowDestroyEvent& event )
{
//	assert (m_activity);
	m_activity.Finish();
	//m_activity.Wait();	
	//delete m_activity;
	//m_activity= NULL;
	CPureDataWrapper::getInstance()->StopPD();
    event.Skip(false);
}


/*!
 * wxEVT_COMMAND_SLIDER_UPDATED event handler for ID_SLIDER_MICVOL
 */

void CPureDataTestW::OnSliderMicvolUpdated( wxCommandEvent& event )
{
	m_activity.setMicInputControl (m_sldMicVol->GetValue());
    event.Skip (false); 
}


/*!
 * wxEVT_COMMAND_SLIDER_UPDATED event handler for ID_SLIDER_OUTVOL
 */

void CPureDataTestW::OnSliderOutvolUpdated( wxCommandEvent& event )
{
	m_activity.setOutputControl (m_sldOutVol->GetValue());
	event.Skip (false); 
}


/*!
 * wxEVT_CLOSE_WINDOW event handler for ID_CPUREDATATESTW
 */

void CPureDataTestW::OnCloseWindow( wxCloseEvent& event )
{
	if (event.CanVeto() && !CPureDataWrapper::getInstance()->getDebugGUIMode()) {
		wxMessageDialog msg(this, _("Save settings as default?"), _("Save settings"), wxYES_NO);
		if (msg.ShowModal()== wxID_YES)
		{
			CAudioProperties* ap= CPureDataWrapper::getInstance()->getAudioProperties ();
			CPureDataWrapper::getInstance()->setAudioProperties (*ap, true);
			delete ap;
		}
	}
	Destroy();
    event.Skip(false);
}

