/////////////////////////////////////////////////////////////////////////////
// Name:       
// Purpose:  
// Author:      Cesar Mauri Loba (cesar at crea-si dot com)
// Modified by: 
// Created:     
// Copyright:   (C) 2010 Cesar Mauri Loba - CREA Software Systems
// 
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
/////////////////////////////////////////////////////////////////////////////

// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

////@begin includes
////@end includes

#include "virtualinstrumentconfig.h"

////@begin XPM images
////@end XPM images


/*!
 * CVirtualInstrumentConfig type definition
 */

IMPLEMENT_CLASS( CVirtualInstrumentConfig, wxMDIChildFrame )


/*!
 * CVirtualInstrumentConfig event table definition
 */

BEGIN_EVENT_TABLE( CVirtualInstrumentConfig, wxMDIChildFrame )

////@begin CVirtualInstrumentConfig event table entries
    EVT_SLIDER( ID_SLIDER_SENSOR_SENSITIVITY, CVirtualInstrumentConfig::OnSliderSensorSensitivityUpdated )

    EVT_CHECKBOX( ID_CHECKBOX_AUTO_DETECT_MOTION_AREA, CVirtualInstrumentConfig::OnCheckboxAutoDetectMotionAreaClick )

    EVT_CHOICE( ID_CHOICE_MUSIC_SCORE, CVirtualInstrumentConfig::OnChoiceMusicScoreSelected )

    EVT_SPINCTRL( ID_SPINCTRL_NOTE_DURATION, CVirtualInstrumentConfig::OnSpinctrlNoteDurationUpdated )

    EVT_CHECKBOX( ID_CHECKBOX_MUTE_PREVIOUS_NOTE, CVirtualInstrumentConfig::OnCheckboxMutePreviousNoteClick )

    EVT_CHOICE( ID_CHOICE_MUSICAL_INSTRUMENT, CVirtualInstrumentConfig::OnChoiceMusicalInstrumentSelected )

////@end CVirtualInstrumentConfig event table entries

END_EVENT_TABLE()

CVirtualInstrumentConfig* CVirtualInstrumentConfig::g_single= NULL;
CVirtualInstrumentConfig* CVirtualInstrumentConfig::getInstance(wxMDIParentFrame* parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style)
{
	if (g_single== NULL) {
		g_single= new CVirtualInstrumentConfig( parent, id, caption, pos, size, style);
	}
	return g_single;
}

/*!
 * CVirtualInstrumentConfig constructors
 */

CVirtualInstrumentConfig::CVirtualInstrumentConfig()
{
    Init();
}

CVirtualInstrumentConfig::CVirtualInstrumentConfig( wxMDIParentFrame* parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
{
    Init();
    Create( parent, id, caption, pos, size, style );
}


/*!
 * cvirtualinstrumentconfig creator
 */

bool CVirtualInstrumentConfig::Create( wxMDIParentFrame* parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
{
////@begin CVirtualInstrumentConfig creation
    wxMDIChildFrame::Create( parent, id, caption, pos, size, style );

    CreateControls();
    if (GetSizer())
    {
        GetSizer()->SetSizeHints(this);
    }
    Centre();
////@end CVirtualInstrumentConfig creation
    return true;
}


/*!
 * CVirtualInstrumentConfig destructor
 */

CVirtualInstrumentConfig::~CVirtualInstrumentConfig()
{
	g_single= NULL;
////@begin CVirtualInstrumentConfig destruction
////@end CVirtualInstrumentConfig destruction
	if (m_activity) delete m_activity;
	m_activity= NULL;
}


/*!
 * Member initialisation
 */

void CVirtualInstrumentConfig::Init()
{
////@begin CVirtualInstrumentConfig member initialisation
    m_sldSensitivity = NULL;
    m_txtSensorSensitivity = NULL;
    m_chkAutoDetectMotionArea = NULL;
    m_choMusicalScore = NULL;
    m_spinNoteDuration = NULL;
    m_chkMutePreviousNote = NULL;
    m_choMusicalInstrument = NULL;
////@end CVirtualInstrumentConfig member initialisation
	m_activity= NULL;
}


/*!
 * Control creation for cvirtualinstrumentconfig
 */

void CVirtualInstrumentConfig::CreateControls()
{    
////@begin CVirtualInstrumentConfig content construction
    CVirtualInstrumentConfig* itemMDIChildFrame1 = this;

    wxBoxSizer* itemBoxSizer2 = new wxBoxSizer(wxVERTICAL);
    itemMDIChildFrame1->SetSizer(itemBoxSizer2);

    wxPanel* itemPanel3 = new wxPanel;
    itemPanel3->Create( itemMDIChildFrame1, ID_PANEL2, wxDefaultPosition, wxDefaultSize, wxSUNKEN_BORDER|wxTAB_TRAVERSAL );
    itemBoxSizer2->Add(itemPanel3, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 0);

    wxStaticBox* itemStaticBoxSizer4Static = new wxStaticBox(itemPanel3, wxID_ANY, wxEmptyString);
    wxStaticBoxSizer* itemStaticBoxSizer4 = new wxStaticBoxSizer(itemStaticBoxSizer4Static, wxVERTICAL);
    itemPanel3->SetSizer(itemStaticBoxSizer4);

    wxStaticBox* itemStaticBoxSizer5Static = new wxStaticBox(itemPanel3, wxID_ANY, _("Sensor"));
    wxStaticBoxSizer* itemStaticBoxSizer5 = new wxStaticBoxSizer(itemStaticBoxSizer5Static, wxVERTICAL);
    itemStaticBoxSizer4->Add(itemStaticBoxSizer5, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);

    wxGridBagSizer* itemGridBagSizer6 = new wxGridBagSizer(0, 0);
    itemGridBagSizer6->SetEmptyCellSize(wxSize(10, 20));
    itemStaticBoxSizer5->Add(itemGridBagSizer6, 0, wxGROW|wxALL, 5);

    wxStaticText* itemStaticText7 = new wxStaticText;
    itemStaticText7->Create( itemPanel3, wxID_STATIC, _("Sensitivity:"), wxDefaultPosition, wxDefaultSize, 0 );
    itemGridBagSizer6->Add(itemStaticText7, wxGBPosition(0, 0), wxGBSpan(1, 1), wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    m_sldSensitivity = new wxSlider;
    m_sldSensitivity->Create( itemPanel3, ID_SLIDER_SENSOR_SENSITIVITY, 0, 0, 100, wxDefaultPosition, wxDefaultSize, wxSL_HORIZONTAL );
    itemGridBagSizer6->Add(m_sldSensitivity, wxGBPosition(0, 1), wxGBSpan(1, 1), wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    m_txtSensorSensitivity = new wxTextCtrl;
    m_txtSensorSensitivity->Create( itemPanel3, ID_TEXTCTRL_SENSOR_SENSITIVITY, wxEmptyString, wxDefaultPosition, wxSize(40, -1), wxTE_READONLY );
    itemGridBagSizer6->Add(m_txtSensorSensitivity, wxGBPosition(0, 2), wxGBSpan(1, 1), wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    m_chkAutoDetectMotionArea = new wxCheckBox;
    m_chkAutoDetectMotionArea->Create( itemPanel3, ID_CHECKBOX_AUTO_DETECT_MOTION_AREA, _("Auto-detect motion area"), wxDefaultPosition, wxDefaultSize, 0 );
    m_chkAutoDetectMotionArea->SetValue(false);
    itemGridBagSizer6->Add(m_chkAutoDetectMotionArea, wxGBPosition(1, 0), wxGBSpan(1, 2), wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxStaticBox* itemStaticBoxSizer11Static = new wxStaticBox(itemPanel3, wxID_ANY, _("Musical notes"));
    wxStaticBoxSizer* itemStaticBoxSizer11 = new wxStaticBoxSizer(itemStaticBoxSizer11Static, wxVERTICAL);
    itemStaticBoxSizer4->Add(itemStaticBoxSizer11, 0, wxGROW|wxALL, 5);

    wxGridBagSizer* itemGridBagSizer12 = new wxGridBagSizer(0, 0);
    itemGridBagSizer12->AddGrowableCol(0);
    itemGridBagSizer12->AddGrowableCol(1);
    itemGridBagSizer12->AddGrowableCol(2);
    itemGridBagSizer12->SetEmptyCellSize(wxSize(10, 20));
    itemStaticBoxSizer11->Add(itemGridBagSizer12, 0, wxGROW|wxALL, 5);

    wxStaticText* itemStaticText13 = new wxStaticText;
    itemStaticText13->Create( itemPanel3, wxID_STATIC, _("Score:"), wxDefaultPosition, wxDefaultSize, 0 );
    itemGridBagSizer12->Add(itemStaticText13, wxGBPosition(0, 0), wxGBSpan(1, 1), wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxArrayString m_choMusicalScoreStrings;
    m_choMusicalScoreStrings.Add(_("Chromatic scale"));
    m_choMusicalScoreStrings.Add(_("Major natural"));
    m_choMusicalScoreStrings.Add(_("Minor natural"));
    m_choMusicalScoreStrings.Add(_("Major pentatonic"));
    m_choMusicalScoreStrings.Add(_("Minor pentatonic"));
    m_choMusicalScoreStrings.Add(_("Blues"));
    m_choMusicalScore = new wxChoice;
    m_choMusicalScore->Create( itemPanel3, ID_CHOICE_MUSIC_SCORE, wxDefaultPosition, wxDefaultSize, m_choMusicalScoreStrings, 0 );
    m_choMusicalScore->SetStringSelection(_("Major pentatonic"));
    itemGridBagSizer12->Add(m_choMusicalScore, wxGBPosition(0, 1), wxGBSpan(1, 2), wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxStaticText* itemStaticText15 = new wxStaticText;
    itemStaticText15->Create( itemPanel3, wxID_STATIC, _("Instrument:"), wxDefaultPosition, wxDefaultSize, 0 );
    itemGridBagSizer12->Add(itemStaticText15, wxGBPosition(1, 0), wxGBSpan(1, 1), wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxStaticText* itemStaticText16 = new wxStaticText;
    itemStaticText16->Create( itemPanel3, wxID_STATIC, _("Note duration:"), wxDefaultPosition, wxDefaultSize, 0 );
    itemGridBagSizer12->Add(itemStaticText16, wxGBPosition(2, 0), wxGBSpan(1, 1), wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    m_spinNoteDuration = new wxSpinCtrl;
    m_spinNoteDuration->Create( itemPanel3, ID_SPINCTRL_NOTE_DURATION, _T("0"), wxDefaultPosition, wxSize(70, -1), wxSP_ARROW_KEYS, 0, 100, 0 );
    itemGridBagSizer12->Add(m_spinNoteDuration, wxGBPosition(2, 1), wxGBSpan(1, 1), wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxStaticText* itemStaticText18 = new wxStaticText;
    itemStaticText18->Create( itemPanel3, wxID_STATIC, _("ms"), wxDefaultPosition, wxDefaultSize, 0 );
    itemGridBagSizer12->Add(itemStaticText18, wxGBPosition(2, 2), wxGBSpan(1, 1), wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    m_chkMutePreviousNote = new wxCheckBox;
    m_chkMutePreviousNote->Create( itemPanel3, ID_CHECKBOX_MUTE_PREVIOUS_NOTE, _("Mute previous"), wxDefaultPosition, wxDefaultSize, 0 );
    m_chkMutePreviousNote->SetValue(false);
    m_chkMutePreviousNote->Show(false);
    m_chkMutePreviousNote->Enable(false);
    itemGridBagSizer12->Add(m_chkMutePreviousNote, wxGBPosition(3, 0), wxGBSpan(1, 2), wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxArrayString m_choMusicalInstrumentStrings;
    m_choMusicalInstrumentStrings.Add(_("1-Acoustic Grand"));
    m_choMusicalInstrumentStrings.Add(_("2-Bright Acoustic"));
    m_choMusicalInstrumentStrings.Add(_("3-Electric Grand"));
    m_choMusicalInstrumentStrings.Add(_("4-Honky-Tonk"));
    m_choMusicalInstrumentStrings.Add(_("5-Electric Piano 1"));
    m_choMusicalInstrumentStrings.Add(_("6-Electric Piano 2"));
    m_choMusicalInstrumentStrings.Add(_("7-Harpsichord"));
    m_choMusicalInstrumentStrings.Add(_("8-Clavinet"));
    m_choMusicalInstrumentStrings.Add(_("9-Celesta"));
    m_choMusicalInstrumentStrings.Add(_("10-Glockenspiel"));
    m_choMusicalInstrumentStrings.Add(_("11-Music Box"));
    m_choMusicalInstrumentStrings.Add(_("12-Vibraphone"));
    m_choMusicalInstrumentStrings.Add(_("13-Marimba"));
    m_choMusicalInstrumentStrings.Add(_("14-Xylophone"));
    m_choMusicalInstrumentStrings.Add(_("15-Tubular Bells"));
    m_choMusicalInstrumentStrings.Add(_("16-Dulcimer"));
    m_choMusicalInstrumentStrings.Add(_("17-Drawbar Organ"));
    m_choMusicalInstrumentStrings.Add(_("18-Percussive Organ"));
    m_choMusicalInstrumentStrings.Add(_("19-Rock Organ"));
    m_choMusicalInstrumentStrings.Add(_("20-Church Organ"));
    m_choMusicalInstrumentStrings.Add(_("21-Reed Organ"));
    m_choMusicalInstrumentStrings.Add(_("22-Accoridan"));
    m_choMusicalInstrumentStrings.Add(_("23-Harmonica"));
    m_choMusicalInstrumentStrings.Add(_("24-Tango Accordian"));
    m_choMusicalInstrumentStrings.Add(_("25-Nylon String Guitar"));
    m_choMusicalInstrumentStrings.Add(_("26-Steel String Guitar"));
    m_choMusicalInstrumentStrings.Add(_("27-Electric Jazz Guitar"));
    m_choMusicalInstrumentStrings.Add(_("28-Electric Clean Guitar"));
    m_choMusicalInstrumentStrings.Add(_("29-Electric Muted Guitar"));
    m_choMusicalInstrumentStrings.Add(_("30-Overdriven Guitar"));
    m_choMusicalInstrumentStrings.Add(_("31-Distortion Guitar"));
    m_choMusicalInstrumentStrings.Add(_("32-Guitar Harmonics"));
    m_choMusicalInstrumentStrings.Add(_("33-Acoustic Bass"));
    m_choMusicalInstrumentStrings.Add(_("34-Electric Bass(finger)"));
    m_choMusicalInstrumentStrings.Add(_("35-Electric Bass(pick)"));
    m_choMusicalInstrumentStrings.Add(_("36-Fretless Bass"));
    m_choMusicalInstrumentStrings.Add(_("37-Slap Bass 1"));
    m_choMusicalInstrumentStrings.Add(_("38-Slap Bass 2"));
    m_choMusicalInstrumentStrings.Add(_("39-Synth Bass 1"));
    m_choMusicalInstrumentStrings.Add(_("40-Synth Bass 2"));
    m_choMusicalInstrumentStrings.Add(_("41-Violin"));
    m_choMusicalInstrumentStrings.Add(_("42-Viola"));
    m_choMusicalInstrumentStrings.Add(_("43-Cello"));
    m_choMusicalInstrumentStrings.Add(_("44-Contrabass"));
    m_choMusicalInstrumentStrings.Add(_("45-Tremolo Strings"));
    m_choMusicalInstrumentStrings.Add(_("46-Pizzicato Strings"));
    m_choMusicalInstrumentStrings.Add(_("47-Orchestral Strings"));
    m_choMusicalInstrumentStrings.Add(_("48-Timpani"));
    m_choMusicalInstrumentStrings.Add(_("49-String Ensemble 1"));
    m_choMusicalInstrumentStrings.Add(_("50-String Ensemble 2"));
    m_choMusicalInstrumentStrings.Add(_("51-SynthStrings 1"));
    m_choMusicalInstrumentStrings.Add(_("52-SynthStrings 2"));
    m_choMusicalInstrumentStrings.Add(_("53-Choir Aahs"));
    m_choMusicalInstrumentStrings.Add(_("54-Voice Oohs"));
    m_choMusicalInstrumentStrings.Add(_("55-Synth Voice"));
    m_choMusicalInstrumentStrings.Add(_("56-Orchestra Hit"));
    m_choMusicalInstrumentStrings.Add(_("57-Trumpet"));
    m_choMusicalInstrumentStrings.Add(_("58-Trombone"));
    m_choMusicalInstrumentStrings.Add(_("59-Tuba"));
    m_choMusicalInstrumentStrings.Add(_("60-Muted Trumpet"));
    m_choMusicalInstrumentStrings.Add(_("61-French Horn"));
    m_choMusicalInstrumentStrings.Add(_("62-Brass Section"));
    m_choMusicalInstrumentStrings.Add(_("63-SynthBrass 1"));
    m_choMusicalInstrumentStrings.Add(_("64-SynthBrass 2"));
    m_choMusicalInstrumentStrings.Add(_("65-Soprano Sax"));
    m_choMusicalInstrumentStrings.Add(_("66-Alto Sax"));
    m_choMusicalInstrumentStrings.Add(_("67-Tenor Sax"));
    m_choMusicalInstrumentStrings.Add(_("68-Baritone Sax"));
    m_choMusicalInstrumentStrings.Add(_("69-Oboe"));
    m_choMusicalInstrumentStrings.Add(_("70-English Horn"));
    m_choMusicalInstrumentStrings.Add(_("71-Bassoon"));
    m_choMusicalInstrumentStrings.Add(_("72-Clarinet"));
    m_choMusicalInstrumentStrings.Add(_("73-Piccolo"));
    m_choMusicalInstrumentStrings.Add(_("74-Flute"));
    m_choMusicalInstrumentStrings.Add(_("75-Recorder"));
    m_choMusicalInstrumentStrings.Add(_("76-Pan Flute"));
    m_choMusicalInstrumentStrings.Add(_("77-Blown Bottle"));
    m_choMusicalInstrumentStrings.Add(_("78-Skakuhachi"));
    m_choMusicalInstrumentStrings.Add(_("79-Whistle"));
    m_choMusicalInstrumentStrings.Add(_("80-Ocarina"));
    m_choMusicalInstrumentStrings.Add(_("81-Square Wave"));
    m_choMusicalInstrumentStrings.Add(_("82-Saw Wave"));
    m_choMusicalInstrumentStrings.Add(_("83-Syn. Calliope"));
    m_choMusicalInstrumentStrings.Add(_("84-Chiffer Lead"));
    m_choMusicalInstrumentStrings.Add(_("85-Charang"));
    m_choMusicalInstrumentStrings.Add(_("86-Solo Vox"));
    m_choMusicalInstrumentStrings.Add(_("87-5th Saw Wave"));
    m_choMusicalInstrumentStrings.Add(_("88-Bass& Lead"));
    m_choMusicalInstrumentStrings.Add(_("89-Fantasia"));
    m_choMusicalInstrumentStrings.Add(_("90-Warm Pad"));
    m_choMusicalInstrumentStrings.Add(_("91-Polysynth"));
    m_choMusicalInstrumentStrings.Add(_("92-Space Voice"));
    m_choMusicalInstrumentStrings.Add(_("93-Bowed Glass"));
    m_choMusicalInstrumentStrings.Add(_("94-Metal Pad"));
    m_choMusicalInstrumentStrings.Add(_("95-Halo Pad"));
    m_choMusicalInstrumentStrings.Add(_("96-Sweep Pad"));
    m_choMusicalInstrumentStrings.Add(_("97-Ice Rain"));
    m_choMusicalInstrumentStrings.Add(_("98-Soundtrack"));
    m_choMusicalInstrumentStrings.Add(_("99-Crystal"));
    m_choMusicalInstrumentStrings.Add(_("100-Atmosphere"));
    m_choMusicalInstrumentStrings.Add(_("101-Brightness"));
    m_choMusicalInstrumentStrings.Add(_("102-Goblin"));
    m_choMusicalInstrumentStrings.Add(_("103-Echo Drops"));
    m_choMusicalInstrumentStrings.Add(_("104-Star Theme"));
    m_choMusicalInstrumentStrings.Add(_("105-Sitar"));
    m_choMusicalInstrumentStrings.Add(_("106-Banjo"));
    m_choMusicalInstrumentStrings.Add(_("107-Shamisen"));
    m_choMusicalInstrumentStrings.Add(_("108-Koto"));
    m_choMusicalInstrumentStrings.Add(_("109-Kalimba"));
    m_choMusicalInstrumentStrings.Add(_("110-Bagpipe"));
    m_choMusicalInstrumentStrings.Add(_("111-Fiddle"));
    m_choMusicalInstrumentStrings.Add(_("112-Shanai"));
    m_choMusicalInstrumentStrings.Add(_("113-Tinkle Bell"));
    m_choMusicalInstrumentStrings.Add(_("114-Agogo"));
    m_choMusicalInstrumentStrings.Add(_("115-Steel Drums"));
    m_choMusicalInstrumentStrings.Add(_("116-Woodblock"));
    m_choMusicalInstrumentStrings.Add(_("117-Taiko Drum"));
    m_choMusicalInstrumentStrings.Add(_("118-Melodic Tom"));
    m_choMusicalInstrumentStrings.Add(_("119-Synth Drum"));
    m_choMusicalInstrumentStrings.Add(_("120-Reverse Cymbal"));
    m_choMusicalInstrumentStrings.Add(_("121-Guitar Fret Noise"));
    m_choMusicalInstrumentStrings.Add(_("122-Breath Noise"));
    m_choMusicalInstrumentStrings.Add(_("123-Seashore"));
    m_choMusicalInstrumentStrings.Add(_("124-Bird Tweet"));
    m_choMusicalInstrumentStrings.Add(_("125-Telephone Ring"));
    m_choMusicalInstrumentStrings.Add(_("126-Helicopter"));
    m_choMusicalInstrumentStrings.Add(_("127-Applause"));
    m_choMusicalInstrumentStrings.Add(_("128-Gunshot"));
    m_choMusicalInstrument = new wxChoice;
    m_choMusicalInstrument->Create( itemPanel3, ID_CHOICE_MUSICAL_INSTRUMENT, wxDefaultPosition, wxDefaultSize, m_choMusicalInstrumentStrings, 0 );
    m_choMusicalInstrument->SetStringSelection(_("0"));
    itemGridBagSizer12->Add(m_choMusicalInstrument, wxGBPosition(1, 1), wxGBSpan(1, 2), wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 5);

////@end CVirtualInstrumentConfig content construction

	assert (m_activity== NULL);
	m_activity= new CVirtualInstrumentActivity();

	m_sldSensitivity->SetRange ((int) m_activity->GetSensorRangePtr()->GetMinLinear(),
 							    (int) m_activity->GetSensorRangePtr()->GetMaxLinear());
	m_sldSensitivity->SetValue ((int) m_activity->GetSensorRangePtr()->GetLinearValue());
	UpdateSliderEvent(ID_SLIDER_SENSOR_SENSITIVITY);

	m_chkAutoDetectMotionArea->SetValue (m_activity->GetAutoDetectMotionArea());

	m_choMusicalInstrument->SetSelection (m_activity->GetMusicalInstrument());

	m_choMusicalScore->SetSelection (m_activity->GetSelectedScore());

	m_spinNoteDuration->SetValue (m_activity->GetNoteDurationPtr()->getMin());
	m_spinNoteDuration->SetRange (m_activity->GetNoteDurationPtr()->getMin(),
	 							m_activity->GetNoteDurationPtr()->getMax());
	m_spinNoteDuration->SetValue (m_activity->GetNoteDurationPtr()->getValue());
	
	m_chkMutePreviousNote->SetValue (m_activity->GetMutePreviousNote());
}

// Send a wxECVT_COMMAND_SLIDER_UPDATED event
void CVirtualInstrumentConfig::UpdateSliderEvent(int id)
{
	wxCommandEvent evt (wxEVT_COMMAND_SLIDER_UPDATED, id);
	GetEventHandler()->ProcessEvent( evt );
}

/*!
 * Should we show tooltips?
 */

bool CVirtualInstrumentConfig::ShowToolTips()
{
    return true;
}

/*!
 * Get bitmap resources
 */

wxBitmap CVirtualInstrumentConfig::GetBitmapResource( const wxString& name )
{
    // Bitmap retrieval
////@begin CVirtualInstrumentConfig bitmap retrieval
    wxUnusedVar(name);
    return wxNullBitmap;
////@end CVirtualInstrumentConfig bitmap retrieval
}

/*!
 * Get icon resources
 */

wxIcon CVirtualInstrumentConfig::GetIconResource( const wxString& name )
{
    // Icon retrieval
////@begin CVirtualInstrumentConfig icon retrieval
    wxUnusedVar(name);
    return wxNullIcon;
////@end CVirtualInstrumentConfig icon retrieval
}


/*!
 * wxEVT_COMMAND_SLIDER_UPDATED event handler for ID_SLIDER_SENSOR_SENSITIVITY
 */

void CVirtualInstrumentConfig::OnSliderSensorSensitivityUpdated( wxCommandEvent& event )
{
	m_activity->GetSensorRangePtr()->SetLinearValue (m_sldSensitivity->GetValue());

	wxString str;
	str.Printf (_T("%.2f"), m_activity->GetSensorRangePtr()->GetExpValue());
	m_txtSensorSensitivity->SetLabel (str);
	
    event.Skip(false);
}

/*!
 * wxEVT_COMMAND_CHECKBOX_CLICKED event handler for ID_CHECKBOX2
 */

void CVirtualInstrumentConfig::OnCheckboxAutoDetectMotionAreaClick( wxCommandEvent& event )
{
	m_activity->SetAutoDetectMotionArea(m_chkAutoDetectMotionArea->IsChecked());
    event.Skip(false);
}

/*!
 * wxEVT_COMMAND_CHOICE_SELECTED event handler for ID_CHOICE_MUSIC_INSTRUMENT
 */

void CVirtualInstrumentConfig::OnChoiceMusicScoreSelected( wxCommandEvent& event )
{
	m_activity->SetSelectedScore (m_choMusicalScore->GetSelection());
    event.Skip(false);
}


/*!
 * wxEVT_COMMAND_SPINCTRL_UPDATED event handler for ID_SPINCTRL_NOTE_DURATION
 */

void CVirtualInstrumentConfig::OnSpinctrlNoteDurationUpdated( wxSpinEvent& event )
{
	m_activity->GetNoteDurationPtr()->setValue((unsigned short) event.GetPosition() );
    event.Skip(false);
}


/*!
 * wxEVT_COMMAND_CHECKBOX_CLICKED event handler for ID_CHECKBOX_MUTE_PREVIOUS_NOTE
 */

void CVirtualInstrumentConfig::OnCheckboxMutePreviousNoteClick( wxCommandEvent& event )
{
	m_activity->SetMutePreviousNote(m_chkMutePreviousNote->IsChecked());
    event.Skip(false);
}


/*!
 * wxEVT_COMMAND_CHOICE_SELECTED event handler for ID_CHOICE_INSTRUMENT
 */

void CVirtualInstrumentConfig::OnChoiceMusicalInstrumentSelected( wxCommandEvent& event )
{
	m_activity->SetMusicalInstrument (m_choMusicalInstrument->GetSelection());
    event.Skip(false);
}

