/////////////////////////////////////////////////////////////////////////////
// Name:        capturethread2.h
// Purpose:		
// Author:      Cesar Mauri Loba (cesar at crea-si dot com)
// Modified by: 
// Created:     
// Copyright:   (C) 2010 Cesar Mauri Loba - CREA Software Systems
// 
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
/////////////////////////////////////////////////////////////////////////////

#ifndef _CAPTURETHREAD2_H_
#define _CAPTURETHREAD2_H_

#include <wx/thread.h>
#include <wx/event.h>
#include <wx/string.h>
#include <cv.h>
#include <list>

// Forward class definitions
class CCamera;
class CCamWindow2;

//
// Abstract class for thread callback
//
class CProcessImage2
{
public:
	virtual void ProcessImage (IplImage *pImage)= 0;
};

//
//
// Event to notify thread termination
//
//
class myThreadEvent : public wxEvent
{
public:
	enum TFlags { STARTED= 1, ENDED= 2, NON_FATAL_ERROR= 4, FATAL_ERROR= 8 };
    
	myThreadEvent( wxString msg= _T("myThreadEvent"), TFlags flags= ENDED, CProcessImage2* who= NULL);
		//: m_msg(msg), m_flags(flags), m_who(who) { }
	
	virtual wxEvent* Clone() const 
	{ 
		return new myThreadEvent(*this); 
	}
	
	const wxString& GetMessage() const { return m_msg; }
	TFlags GetFlags() const { return m_flags; }
	CProcessImage2* GetWho () { return m_who; } 
private:
	wxString m_msg;
	TFlags m_flags;
	CProcessImage2* m_who;

	DECLARE_DYNAMIC_CLASS(myThreadEvent)
};

BEGIN_DECLARE_EVENT_TYPES()
	DECLARE_EVENT_TYPE( wxEVT_MY_THREAD_EVENT, -1 )
END_DECLARE_EVENT_TYPES()

typedef void (wxEvtHandler::*myThreadEventFunction)(myThreadEvent&);
#define EVT_MY_THREAD(fn) \
    DECLARE_EVENT_TABLE_ENTRY( wxEVT_MY_THREAD_EVENT, wxID_ANY, wxID_ANY, \
    (wxObjectEventFunction) (wxEventFunction) &fn, (wxObject *) NULL ),

//#define EVT_MY_THREAD(id, fn) \
//    DECLARE_EVENT_TABLE_ENTRY( wxEVT_MY_THREAD_EVENT, id, -1, \
//    (wxObjectEventFunction) (wxEventFunction) \
//    wxStaticCastEvent( myThreadEventFunction, & fn ), (wxObject *) NULL ),

//
// End event stuff
//


class CCaptureThread2 : public wxThread
{
public:
	CCaptureThread2 (CCamera *pCamera, wxThreadKind kind = wxTHREAD_DETACHED, wxEvtHandler* evtDest= NULL);
	~CCaptureThread2 ();

	// Thread entry point
	virtual wxThread::ExitCode Entry();

	void Finish() { m_Life= false; }

	// Return the number of processors after
	int RegisterImageProcessor (CProcessImage2* iProcessor); 
	int UnregisterImageProcessor (CProcessImage2* iProcessor);
	

	// Return the number of drawer after
	int RegisterImageDrawer (CCamWindow2* iDrawer); 
	int UnregisterImageDrawer (CCamWindow2* iDrawer);

	int GetRegisteredImageProcessors ();
	int GetRegisteredImageDrawers ();

private:
	// All these 4 functions return the number of processors/drawers presents after

	int DoRegisterImageProcessor (CProcessImage2* iProcessor); 
	int DoUnregisterImageProcessor (CProcessImage2* iProcessor);

	int DoRegisterImageDrawer (CCamWindow2* iDrawer); 
	int DoUnregisterImageDrawer (CCamWindow2* iDrawer);

	/// TO DELETE
	
	///

	typedef std::list<CProcessImage2 *> TImageProcessorList;
	typedef TImageProcessorList::iterator TImageProcessorListIterator;

	typedef std::list<CCamWindow2 *> TImageDrawerList;
	typedef TImageDrawerList::iterator TImageDrawerListIterator; 


	volatile bool m_Life;
	CCamera *m_pCamera;
	
	wxEvtHandler* m_eventDest;

	TImageProcessorList m_ImageProcessorList;
	TImageDrawerList	m_ImageDrawerList;
	wxCriticalSection	m_mutex;
};

#endif

